@@ -10,6 +10,7 @@ use anyhow::Result;
 use log::*;
 use nix::sched::CpuSet;
 use std::marker::PhantomData;
+use std::mem::forget;
 use std::rc::Rc;
 use std::sync::{Arc, Mutex};
 
@@ -32,6 +33,33 @@ impl CoreGroup {
             inner: CoreGroupInner::Cores(cores),
         }
     }
+    pub fn reserve(&self) -> Vec<CoreIndex> {
+        match &self.inner {
+            CoreGroupInner::AnyCore => {
+                vec![] // no dedicated cores
+            }
+            CoreGroupInner::Cores(cores) => cores
+                .iter()
+                .map(|x| x.lock().unwrap())
+                .map(|x| {
+                    let id = CoreIndex::new(x.get_raw());
+                    forget(x);
+                    id
+                })
+                .collect(),
+        }
+    }
+    pub fn bind_nth(&self, index: usize) -> Result<Cleanup> {
+        match &self.inner {
+            CoreGroupInner::AnyCore => Ok(Cleanup::new(None)),
+            CoreGroupInner::Cores(cores) => {
+                let core = cores.get(index).with_context(|| {
+                    format!("Could not find {}th core in the group {:?}", index, cores)
+                })?;
+                core.lock().unwrap().bind()
+            }
+        }
+    }
 }
 enum CoreGroupInner {
     AnyCore,
@@ -52,20 +80,6 @@ impl Debug for CoreGroup {
     }
 }
 
-impl CoreGroup {
-    pub fn bind_nth(&self, index: usize) -> Result<Cleanup> {
-        match &self.inner {
-            CoreGroupInner::AnyCore => Ok(Cleanup::new(None)),
-            CoreGroupInner::Cores(cores) => {
-                let core = cores.get(index).with_context(|| {
-                    format!("Could not find {}th core in the group {:?}", index, cores)
-                })?;
-                core.lock().unwrap().bind()
-            }
-        }
-    }
-}
-
 #[derive(Clone, Copy, Debug, PartialEq)]
 pub struct CoreIndex(usize);
 
@@ -95,6 +109,9 @@ impl<'a> Cleanup<'a> {
             _phantom: Default::default(),
         }
     }
+    pub fn detach(mut self) {
+        self.prior_state.take();
+    }
 }
 
 impl<'a> Drop for Cleanup<'a> {
